<?php
/**
 * Google XML Sitemap Feed Cron Script
 *
 * The Google sitemap service was announced on 2 June 2005 and represents
 * a huge development in terms of crawler technology.  This contribution is
 * designed to create the sitemap XML feed per the specification delineated 
 * by Google.  This cron script will call the code to create the scripts and 
 * eliminate the session auto start issues. 
 * @package Google-XML-Sitemap-Feed
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @version 1.13
 * @link http://www.oscommerce-freelancers.com/ osCommerce-Freelancers
 * @link http://www.google.com/webmasters/sitemaps/docs/en/about.html About Google Sitemap 
 * @copyright Copyright 2005, Bobby Easland 
 * @author Bobby Easland 
 * @filesource
 */

	chdir('../');
 
 error_reporting(0);
 ini_set('display_errors','0'); 
  if (! extension_loaded('mysqli')) {
    echo 'mysqli is required but can\'t be found. Aborting!!!';
    exit;
  }  

	/**
	 * Option to compress the files
	 */
	define('GOOGLE_SITEMAP_COMPRESS', 'false');
	define('GOOGLE_SITEMAP_PROD_CHANGE_FREQ', 'weekly');
	define('GOOGLE_SITEMAP_CAT_CHANGE_FREQ', 'weekly');
	define('GOOGLE_SITEMAP_IMAGES_CHANGE_FREQ', 'weekly');
	define('GOOGLE_SITEMAP_MAN_CHANGE_FREQ', 'weekly');
	define('GOOGLE_SITEMAP_SPECIALS_CHANGE_FREQ', 'weekly');
	define('GOOGLE_SITEMAP_PAGES_CHANGE_FREQ', 'weekly'); 
 define('GOOGLE_SITEMAP_TOPICS_CHANGE_FREQ', 'weekly');
	
 define('DIR_WS_CATALOG', DIR_WS_HTTP_CATALOG);

  include_once('includes/application_top.php');
  
  if (! empty(GOOGLE_XML_SITEMAP_SECURE_IP)) {
     $safe_ips = explode(',', GOOGLE_XML_SITEMAP_SECURE_IP);
  
     if (! in_array($_SERVER['REMOTE_ADDR'], $safe_ips)) {
        header("location:http:127.0.0.1");
     }
  }
 
  function exclude_robots($file = 'robots.txt'){
    if (file_exists($file)) {
       $file_text = file_get_contents($file);
       preg_match_all("(Disallow: /((.*)\.php))", $file_text, $matches_exclude);
     
       return $matches_exclude[1];
    }
    return array();
  }
 
$mapfile = 'sitemap.class.php';
require_once('googlesitemap/' . $mapfile);

$file = file_get_contents('includes/application_top.php');
$version = "<head>";
if (strpos($file, 'v2.3') !== FALSE) {
  $version = "template_top.php";
}

$domainArray = array();
$domainArray[] = HTTP_SERVER . DIR_WS_HTTP_CATALOG;
$map_type = 'standard';
 
foreach ($domainArray as $domain_name) {
   $google = new GoogleSitemap(DB_SERVER, DB_SERVER_USERNAME, DB_DATABASE, DB_SERVER_PASSWORD, $version, $domain_name, $map_type);

   $showImages = false;
   $showManufacturers = false;
   $showPages = false;
   $showSpecials = false;
   $showTopics = false;
   $submit = true;
?>
<style>.red { color:red; } </style>
<?php
   echo '<pre>';
   if ($google->GenerateProductSitemap()){
    if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Product Sitemap Successfully' . "\n\n";
   } else {
    $submit = false;
    echo 'ERROR: Google Product Sitemap Generation FAILED!' . "\n\n";
   }

   if ($google->GenerateCategorySitemap()){
    if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Category Sitemap Successfully' . "\n\n";
   } else {
    $submit = false;
    echo 'ERROR: Google Category Sitemap Generation FAILED!' . "\n\n";
   }

   if (GOOGLE_XML_SITEMAP_CREATE_IMAGES == 'true') {
     if ($google->GenerateImagesSitemap()){
         $showImages = true;
         if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Images Sitemap Successfully' . "\n\n";
     } else {
         $submit = false;
         echo 'ERROR: Google Images Sitemap Generation FAILED!' . "\n\n";
     }
   }
    
   if (GOOGLE_XML_SITEMAP_CREATE_MANU == 'true') {
     if ($google->GenerateManufacturerSitemap()){
           $showManufacturers = true;
      if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Manufacturers Sitemap Successfully' . "\n\n";
     } else {
       $manufacturers_query = tep_db_query("select manufacturers_id from manufacturers limit 1");
       if (tep_db_num_rows($manufacturers_query) > 0) {
           $submit = false;
           echo 'ERROR: Google Manufacturers Sitemap Generation FAILED!' . "\n\n";
       } else {
           echo 'Google Sitemap Manufacturers not generated - no Manufacturers found!' . "\n\n";
       }
     }
   }
   if (GOOGLE_XML_SITEMAP_CREATE_PAGES == 'true') {
     if ($google->GeneratePagesSitemap()){
         $showPages = true;
         if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Pages Sitemap Successfully' . "\n\n";
     } else {
         $submit = false;
         echo 'ERROR: Google Pages Sitemap Generation FAILED!' . "\n\n";
     }
   }

   if (GOOGLE_XML_SITEMAP_CREATE_SPECIALS == 'true') {
     if ($google->GenerateSpecialsSitemap($languages_id)){
           $showSpecials = true;
      if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Specials Sitemap Successfully' . "\n\n";
     } else {
       $specials_query = tep_db_query("select p.products_id from products p, products_description pd, specials s where p.products_status = '1' and p.products_id = s.products_id and pd.products_id = s.products_id and pd.language_id = '" . (int)$languages_id . "' and s.status = '1' limit 1");
       if (tep_db_num_rows($specials_query) > 0) {
        $submit = false;
        echo 'ERROR: Google Specials Sitemap Generation FAILED!' . "\n\n";
       } else {
        echo 'Google Sitemap Specials not generated - no specials found!' . "\n\n";
       }
     }
   }

   if (GOOGLE_XML_SITEMAP_CREATE_TOPICS == 'true') {
     if ($google->GenerateTopicsSitemap($languages_id)){
           $showTopics = true;
      if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Topics Sitemap Successfully' . "\n\n";
     } else {
      $submit = false;
      echo 'ERROR: Google Topics Sitemap Generation FAILED!' . "\n\n";
     }
   }

   if ($google->GenerateSitemapIndex()){
    if (GOOGLE_XML_SITEMAP_SHOW_DIAGNOSTIC == 'true') echo 'Generated Google Sitemap Index Successfully' . "\n\n";
   } else {
    $submit = false;
    echo 'ERROR: Google Sitemap Index Generation FAILED!' . "\n\n";
   }

   if ($submit){
      if ($map_type == 'standard') { //only show once
          echo '<span class="red"><strong>CONGRATULATIONS! All files generated successfully.</strong></span>' . "\n\n";
          echo 'If you have not already submitted the sitemap index to Google or Bing click the link(s) below.' . "\n";
          echo '<span class="red">Before you do I HIGHLY recommend that you view the XML files to make sure the data is correct.</span>' . "\n\n";
          
          if (isset($_SERVER['HTTP_USER_AGENT'])) { //not a ceon job
             echo "\t" . '<a href="' . $google->GenerateSubmitURL('google') . '">Notify Google</a>';
             echo "\t" . '<a href="' . $google->GenerateSubmitURL('bing') . '">Notify Bing</a>' . "\n\n";
          } else {
             echo $google->GenerateSubmitURL('google') . "\n\n"; 
             echo $google->GenerateSubmitURL('bing') . "\n\n";           
          }          
          echo str_repeat("-=", 60) ."\n";
          
          echo 'For your convenience here is the <strong>CRON command</strong> for your site:' . "\n\n";
          echo "\t" . '<strong>php ' . dirname($_SERVER['SCRIPT_FILENAME']) . '/index.php</strong>' . "\n\n";
          echo str_repeat("-=", 60) ."\n";
          
          echo 'The following should be added to your Google and Bing Webmaster Tools accounts.' . "\n";
          echo 'They only require the index to be submitted but listing all of them will make it easier to troubleshoot problems.' . "\n\n";
          echo "\t" . 'sitemap: ' . $google->base_url . 'sitemapindex.xml' . "\n";
          echo "\n";
          echo str_repeat("-=", 60) ."\n";
          
          echo 'If you have a robots.txt file (located in the root directory of your site), add this to the end of it:' . "\n\n";
          echo "\t" . 'sitemap: ' . $google->base_url . 'sitemapindex.xml' . "\n";
          echo "\n";
      }
      echo str_repeat("-=", 60) ."\n";
                     
      echo 'Here are the map files for the URL '. $google->base_url . "\n\n";     
      echo ($map_type == 'standard' ? 'sitemapindex.xml' : 'sitemap_mobileindex.xml') . "\n";
      echo ($map_type == 'standard' ? 'sitemapproducts.xml' : 'sitemap_mobileproducts.xml') . "\n";
      echo ($map_type == 'standard' ? 'sitemapcategories.xml' : 'sitemap_mobilecategories.xml') . "\n";

      if (GOOGLE_XML_SITEMAP_CREATE_IMAGES == 'true' && $showImages)
       echo ($map_type == 'standard' ? 'sitemapimages.xml' : 'sitemap_mobileimages.xml') . "\n";
      
      if (GOOGLE_XML_SITEMAP_CREATE_MANU == 'true' && $showManufacturers)
       echo ($map_type == 'standard' ? 'sitemapmanufacturers.xml' : 'sitemap_mobilemanufacturers.xml') . "\n";

      if (GOOGLE_XML_SITEMAP_CREATE_PAGES == 'true')
       echo ($map_type == 'standard' ? 'sitemappages.xml' : 'sitemap_mobilepages.xml') . "\n";

      if (GOOGLE_XML_SITEMAP_CREATE_SPECIALS == 'true' && $showSpecials)
       echo ($map_type == 'standard' ? 'sitemapspecials.xml' : 'sitemap_mobilespecials.xml') . "\n";

      if (GOOGLE_XML_SITEMAP_CREATE_TOPICS == 'true' && $showTopics)
       echo ($map_type == 'standard' ? 'sitemaptopics.xml' : 'sitemap_mobiletopics.xml') . "\n";
   } else {
    print_r($google->debug);
   }

   echo '</pre>';
   echo '<br /><br /><br />';

   if (count($domainArray) > 1) {
       $map_type = 'mobile'; //for the next pass
       define ('MOBILE_SESSION_GOOGLE', true); //trick Ultimate SEO into using the mobile url
   }
}

echo '<span class="red"><strong>Completed!</strong></span><br /><br />';
 